SystemOrganization addCategory: #'Phidget-Examples'!

Object subclass: #PhdExIfKitLED
	instanceVariableNames: 'ifKit running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLED commentStamp: 'pt 7/6/2009 16:54' prior: 0!
This is an example for the usage of an InterfaceKit (ifKit).
We expect that there are LED connected to the digital output of the ifKit.

execute this in a workspace:
	led := PhdExIfKitLED new. "creates a new LEDExample"
	led start.				"starts the animation - opens a connection to the ifKit"
	led stop.				"stops the animation - closes the connection - recommended if you want to use this Phidget later. If the connection stays open, the Phidget can not be used somewhere else."!

!PhdExIfKitLED methodsFor: 'initialize' stamp: 'pt 6/30/2009 11:12'!
initialize

	super initialize.
	ifKit := PhdCrInterfaceKit new.! !

!PhdExIfKitLED methodsFor: 'events' stamp: 'fs 7/24/2009 15:28'!
process

	[running] whileTrue: [
		ifKit outputs size = 0 ifFalse: [
			self setLEDState].
		(Delay forMilliseconds: self waitTime) wait]! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:30'!
setLEDState

	self toggle: {(1 to: ifKit outputs size) atRandom}.! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:30'!
start

	ifKit openAny.
	running := true.
	[self process] fork.! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:30'!
stop

	running := false.
	self turnAllOff.
	ifKit close.! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:32'!
toggle: anArray
	"toggle state if needed for all led"
	
	anArray do: [: i |
		(ifKit outputs at: i) value: (ifKit outputs at: i) value not]! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:32'!
turnAllOff

	(1 to: ifKit outputs size) do: [:i | (ifKit outputs at: i) value: false].! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:33'!
turnAllOn

	(1 to: ifKit outputs size) do: [:i | (ifKit outputs at: i) value: true].! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:33'!
turnOff: anArray

	anArray do: [:i | (ifKit outputs at: i) value: false].! !

!PhdExIfKitLED methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:33'!
turnOn: anArray

	anArray do: [:i | (ifKit outputs at: i) value: true].! !

!PhdExIfKitLED methodsFor: 'accessing' stamp: 'fs 7/24/2009 15:27'!
waitTime

	^ 200! !

PhdExIfKitLED subclass: #PhdExIfKitLEDCircles
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDCircles methodsFor: 'initialize' stamp: 'fs 7/24/2009 15:38'!
initialize

	super initialize.
	state := #1.! !

!PhdExIfKitLEDCircles methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:38'!
setLEDState

	self turnAllOff.
	state =  1 ifTrue: [                           state :=  2. ^ self].
	state =  2 ifTrue: [self turnOn: {1. 4. 5. 8}. state :=  3. ^ self].
	state =  3 ifTrue: [self turnOn: {2. 3. 6. 7}. state :=  1. ^ self].! !

!PhdExIfKitLEDCircles methodsFor: 'accessing' stamp: 'fs 7/24/2009 15:38'!
waitTime

	^ 150! !

PhdExIfKitLED subclass: #PhdExIfKitLEDNothing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDNothing commentStamp: 'pt 7/6/2009 16:57' prior: 0!
Does nothing, but turn all LEDs on. This may be used for own experiments. Edit setLEDstate to define what happens every <self waitTime> milliseconds. (You may also edit my method waitTime ;))

This example is based on PhdExIfKitLED. Have a look on the comment of that class for further information.!

!PhdExIfKitLEDNothing methodsFor: 'toggle state' stamp: 'pt 7/1/2009 12:01'!
setLEDState
	self turnAllOn.! !

!PhdExIfKitLEDNothing methodsFor: 'accessing' stamp: 'fs 7/24/2009 15:38'!
waitTime

	^ 1000! !

PhdExIfKitLED subclass: #PhdExIfKitLEDSpiral
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDSpiral commentStamp: 'pt 7/6/2009 16:55' prior: 0!
Makes the LEDs flash in a spiral.

This class expects the LED ordered as the following:
    4
    3
8 7   6 5
    2
    1

This example is based on PhdExIfKitLED. Have a look on the comment of that class for further information.!

!PhdExIfKitLEDSpiral methodsFor: 'initialize' stamp: 'pt 6/30/2009 12:01'!
initialize
	super initialize.
	state := #1.! !

!PhdExIfKitLEDSpiral methodsFor: 'toggle state' stamp: 'pt 6/30/2009 12:05'!
setLEDState
	
	state =  1 ifTrue: [self turnOn: {1.}. state :=  2. ^ self].
	state =  2 ifTrue: [self turnOn: {8.}. state :=  3. ^ self].
	state =  3 ifTrue: [self turnOn: {4.}. state :=  4. ^ self].
	state =  4 ifTrue: [self turnOn: {5.}. state :=  5. ^ self].
	state =  5 ifTrue: [self turnOn: {2.}. state :=  6. ^ self].
	state =  6 ifTrue: [self turnOn: {7.}. state :=  7. ^ self].
	state =  7 ifTrue: [self turnOn: {3.}. state :=  8. ^ self].
	state =  8 ifTrue: [self turnOn: {6.}. state :=  9. ^ self].
	
	state =  9 ifTrue: [self turnOff: {1.}. state := 10. ^ self].
	state = 10 ifTrue: [self turnOff: {8.}. state := 11. ^ self].
	state = 11 ifTrue: [self turnOff: {4.}. state := 12. ^ self].
	state = 12 ifTrue: [self turnOff: {5.}. state := 13. ^ self].
	state = 13 ifTrue: [self turnOff: {2.}. state := 14. ^ self].
	state = 14 ifTrue: [self turnOff: {7.}. state := 15. ^ self].
	state = 15 ifTrue: [self turnOff: {3.}. state := 16. ^ self].
	state = 16 ifTrue: [self turnOff: {6.}. state :=  1. ^ self].! !

!PhdExIfKitLEDSpiral methodsFor: 'accessing' stamp: 'pt 6/30/2009 15:14'!
waitTime
	^ 150! !

PhdExIfKitLED subclass: #PhdExIfKitLEDToggle
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExIfKitLEDToggle commentStamp: 'pt 7/6/2009 16:58' prior: 0!
Makes the two LED axes alternately flash.

This class expects the LED ordered as the following:
    4
    3
8 7   6 5
    2
    1

This example is based on PhdExIfKitLED. Have a look on the comment of that class for further information.!

!PhdExIfKitLEDToggle methodsFor: 'initialize' stamp: 'fs 7/24/2009 15:41'!
initialize

	super initialize.
	state := #1.! !

!PhdExIfKitLEDToggle methodsFor: 'toggle state' stamp: 'fs 7/24/2009 15:41'!
setLEDState

	self turnAllOff.
	state = #1 ifTrue: [self turnOn: {1. 2. 3. 4.}. state := 2. ^ self].
	state = #2 ifTrue: [self turnOn: {5. 6. 7. 8.}. state := 1. ^ self].! !

Object subclass: #PhdExListItem
	instanceVariableNames: 'nextLink value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExListItem commentStamp: 'pt 7/6/2009 16:46' prior: 0!
This is a class used in the PhidgetLab RFID example.
See PhdExRFIDPaint or PhdExTagMap for more information about the example.!

!PhdExListItem class methodsFor: 'as yet unclassified' stamp: 'lk 7/2/2009 22:36'!
new: anObject
	"create a new list item with given object as content."
	
	^ self new
		value: anObject.! !

!PhdExListItem methodsFor: 'accessing' stamp: 'lk 7/2/2009 21:52'!
nextLink

	^ nextLink! !

!PhdExListItem methodsFor: 'accessing' stamp: 'lk 7/2/2009 21:52'!
nextLink: anObject

	nextLink := anObject.! !

!PhdExListItem methodsFor: 'accessing' stamp: 'lk 7/2/2009 21:53'!
value: anObject

	value := anObject.! !

Object subclass: #PhdExLunarLander
	instanceVariableNames: 'motorControl interfaceKit lcd leftRocket rightRocket state leftPlayer rightPlayer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExLunarLander methodsFor: 'phidgets' stamp: 'lk 8/22/2009 20:55'!
allPhidgetsAttached

	^ motorControl isAttached & interfaceKit isAttached & lcd isAttached
		and: [ motorControl motors size = 2 ]
		and: [ interfaceKit sensors size = 8 ]
		and: [ interfaceKit inputs size = 8 ]
		and: [ interfaceKit outputs size = 8 ]! !

!PhdExLunarLander methodsFor: 'phidgets' stamp: 'lk 8/22/2009 20:59'!
closePhidgets

	motorControl close.
	interfaceKit close.
	lcd close.! !

!PhdExLunarLander methodsFor: 'controllers' stamp: 'lk 8/22/2009 20:43'!
controllerInStartingPosition: rocketId

	| controlSensor |
	rocketId = #left ifTrue: [ controlSensor := self leftControlSensor ].
	rocketId = #right ifTrue: [ controlSensor := self rightControlSensor ].
	
	^ (interfaceKit sensors at: controlSensor) value // 100 = 0! !

!PhdExLunarLander methodsFor: 'controllers' stamp: 'lk 8/22/2009 20:43'!
controllersInStartingPosition

	^ (self controllerInStartingPosition: #left) and: [ (self controllerInStartingPosition: #right) ]! !

!PhdExLunarLander methodsFor: 'controllers' stamp: 'lk 8/20/2009 22:20'!
currentControlValue

	interfaceKit isAttached ifFalse: [ ^ 0 ].
	^ (interfaceKit sensors at: 8) value! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/22/2009 21:12'!
doCountDown

	| delay |
	delay := Delay forMilliseconds: 800.

	self setStateLEDs: #( false false true false false true ).
	self showInfo: 'Ready ...'.
	delay wait.
	self setStateLEDs: #( false true true false true true ).
	self showInfo: 'Steady ...'.
	delay wait.
	self setStateLEDs: #( true false false true false false ).
	self showInfo: 'Go!!'.
	delay wait.
	
	self triggerEvent: #CountDownDone.! !

!PhdExLunarLander methodsFor: 'controllers' stamp: 'lk 8/22/2009 19:27'!
doRoundControlling

	| controlValue |
	controlValue := self currentControlValue.
	
	controlValue >= 666 ifTrue: [
		"stop"
		(state ~= #initialized and: [ state ~= #stopped ]) ifTrue: [ self switchToState: #stopped ].
		self setStateLEDs: #( false false false false false false ).
		^ self
	].
	controlValue <= 275 ifTrue: [
		"start round"
		state = #reset ifTrue: [ self switchToState: #waitForControllersInStartingPosition ].
		^ self
	].
	
	"reset"
	state = #stopped ifTrue: [ self switchToState: #waitForAttach ].! !

!PhdExLunarLander methodsFor: 'velocities' stamp: 'lk 8/21/2009 22:57'!
explodeLimit

	^ -18! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 23:15'!
greenLEDFor: rocketId

	rocketId = #left ifTrue: [ ^ self leftGreenLED ].
	rocketId = #right ifTrue: [ ^ self rightGreenLED ].! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/22/2009 19:38'!
ifAllPhidgetsAttachedReset

	Transcript show: 'ifallphidgetsattachedreset'; cr.

	self allPhidgetsAttached ifTrue: [
		self switchToState: #resetting.
		^ true
	].

	^ false! !

!PhdExLunarLander methodsFor: 'controllers' stamp: 'lk 8/22/2009 20:07'!
ifControllersInStartingPositionStartRound

	self controllersInStartingPosition ifTrue: [
		self switchToState: #countDown.
	].! !

!PhdExLunarLander methodsFor: 'controllers' stamp: 'lk 8/21/2009 22:08'!
ifPlayersReadyStartRound
	"if controller is at 'start round', start a round =)"
	
	self currentControlValue >= 666 ifTrue: [
		self switchToState: #waitForControllersInStartingPosition
	].! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:58'!
ifRocketsResetSwitchToResetState

	leftRocket isReset & rightRocket isReset ifTrue: [
		self switchToState: #reset.
		^ true
	].

	^ false! !

!PhdExLunarLander methodsFor: 'initialization' stamp: 'pt 8/28/2009 09:17'!
initialize

	super initialize.
	
	motorControl := PhdCrMotorControl new.
	interfaceKit := PhdCrInterfaceKit new.
	lcd := PhdCrTextLCD new.
	
	leftPlayer := 0.
	rightPlayer := 0.
	
	leftRocket := PhdExLunarLanderRocket newAs: #left inverted: true.
	rightRocket := PhdExLunarLanderRocket newAs: #right inverted: false.
	
	state := #initialized.! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/22/2009 20:38'!
ledStateProcess

	| longDelay shortDelay delay blinkState |
	longDelay := Delay forMilliseconds: 400.
	shortDelay := Delay forMilliseconds: 200.
	delay := longDelay.
	blinkState := 0.
	
	[ state ~= #initialised ] whileTrue: [
		
		state caseOf: {
			[#stopped] -> [ "the last winner is shown (if any). see rocketFinished:withVelocity:" ].
			[#waitForAttach] -> [
				blinkState = 0 ifTrue: [ self setStateLEDs: #( false true false false true false ) ].
				blinkState = 1 ifTrue: [ self setStateLEDs: #( false false false false false false ) ].
				blinkState := (blinkState + 1) \\ 2.
			].
			[#resetting] -> [
				blinkState = 0 ifTrue: [ self setStateLEDs: #( true true true true true true ) ].
				blinkState = 1 ifTrue: [ self setStateLEDs: #( false false false false false false ) ].
				blinkState := (blinkState + 1) \\ 2.
			].
			[#reset] -> [
				blinkState = 0 ifTrue: [ self setStateLEDs: #( true false false true false false ) ].
				blinkState = 1 ifTrue: [ self setStateLEDs: #( false false false false false false ) ].
				blinkState := (blinkState + 1) \\ 2.
			].
			[#waitForControllersInStartingPosition] -> [
				blinkState = 0 ifTrue: [ self showControllersInStartingPosition ].
				blinkState = 1 ifTrue: [ self setStateLEDs: #( false false false false false false ) ].
				blinkState := (blinkState + 1) \\ 2.
			].
			[#countDown] -> [ self doCountDown	].
			[#roundStarted] -> [ self showRocketVelocities ].
		} otherwise: [ ^ self ].
		
		delay wait.
	].! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 13:32'!
leftBaseInput
	^ 1! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 20:09'!
leftControlSensor
	^ 6! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:32'!
leftGreenLED
	^ 6! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 13:32'!
leftMotor

	^ 1! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:33'!
leftRedLED
	^ 4! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:33'!
leftYellowLED
	^ 5! !

!PhdExLunarLander methodsFor: 'velocities' stamp: 'lk 8/21/2009 23:16'!
maxUncertaintyLimit

	^ -40! !

!PhdExLunarLander methodsFor: 'velocities' stamp: 'lk 8/21/2009 23:16'!
minUncertaintyLimit

	^ -13! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 22:12'!
onSensorChange

	self doRoundControlling.
	state = #waitForControllersInStartingPosition ifTrue: [
		self ifControllersInStartingPositionStartRound.
	].! !

!PhdExLunarLander methodsFor: 'phidgets' stamp: 'lk 8/22/2009 20:59'!
openPhidgets

	motorControl openAny.
	interfaceKit openAny.
	lcd openAny.! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/22/2009 20:14'!
otherPlayerFor: id

	id = #left ifTrue: [ ^ #right ].
	id = #right ifTrue: [ ^ #left ].! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/21/2009 23:28'!
otherRocketFor: id

	id = #left ifTrue: [ ^ rightRocket ].
	id = #right ifTrue: [ ^ leftRocket ].! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/22/2009 20:52'!
player: rocketId lost: points
	
	self setPlayerPoints: rocketId to: (self playerPointsFor: rocketId) + points.
	self setStateLeds: #( false false true ) for: rocketId.! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/21/2009 23:22'!
player: rocketId won: points
	
	self setPlayerPoints: rocketId to: (self playerPointsFor: rocketId) + points.
	self setStateLeds: #( true false false ) for: rocketId.! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/22/2009 20:48'!
playerGameOver: rocketId

	self setStateLeds: #( false false true ) for: rocketId.! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/21/2009 23:21'!
playerPointsFor: rocketId

	rocketId = #left ifTrue: [ ^ leftPlayer ].
	rocketId = #right ifTrue: [ ^ rightPlayer ].! !

!PhdExLunarLander methodsFor: 'points' stamp: 'lk 8/22/2009 20:52'!
pointsForDoubleWinning

	^ 5! !

!PhdExLunarLander methodsFor: 'points' stamp: 'lk 8/22/2009 20:52'!
pointsForLosing

	^ -1! !

!PhdExLunarLander methodsFor: 'points' stamp: 'lk 8/22/2009 20:52'!
pointsForWinning

	^ 2! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 23:15'!
redLEDFor: rocketId

	rocketId = #left ifTrue: [ ^ self leftRedLED ].
	rocketId = #right ifTrue: [ ^ self rightRedLED ].! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 22:11'!
registerController

	interfaceKit
		when: #SensorChange
		send: #onSensorChange
		to: self.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 22:20'!
registerCountDownDoneEvent

	self
		when: #CountDownDone
		send: #switchToState:
		to: self
		with: #roundStarted.
	"this event will be triggered in the LED state process."! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/22/2009 14:20'!
registerFlyingDoneEvents

	leftRocket
		when: #FlyingDone
		send: #rocketFinished:withVelocity:
		to: self.
	rightRocket
		when: #FlyingDone
		send: #rocketFinished:withVelocity:
		to: self.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/22/2009 19:40'!
registerOnAllAttachedReset
	"if all phidgets are already attached, reset now.
	else register the events and try to reset then."
	
	Transcript show: 'registerOnAllAttachedReset'; cr.
	
	self ifAllPhidgetsAttachedReset ifFalse: [
		Transcript show: 'registerOnAllAttachedReset - false'; cr.
		motorControl
			when: #Attach
			send: #ifAllPhidgetsAttachedReset
			to: self.
		interfaceKit
			when: #Attach
			send: #ifAllPhidgetsAttachedReset
			to: self.
	].! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:48'!
registerOnDetachStopRockets

	motorControl
		when: #Detach
		send: #switchToState:
		to: self
		with: #stopped.
	interfaceKit
		when: #Detach
		send: #switchToState:
		to: self
		with: #stopped.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:57'!
registerResetDoneEvents

	self ifRocketsResetSwitchToResetState ifFalse: [
		leftRocket
			when: #ResetDone
			send: #ifRocketsResetSwitchToResetState
			to: self.
		rightRocket
			when: #ResetDone
			send: #ifRocketsResetSwitchToResetState
			to: self.
	].! !

!PhdExLunarLander methodsFor: 'rocket actions' stamp: 'lk 8/22/2009 19:37'!
resetRockets

	[ leftRocket reset ] fork.
	[ rightRocket reset ] fork.! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 13:33'!
rightBaseInput
	^ 2! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 20:10'!
rightControlSensor
	^ 5! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:33'!
rightGreenLED
	^ 3! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 13:32'!
rightMotor
	^ 2! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:33'!
rightRedLED
	^ 1! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:33'!
rightYellowLED
	^ 2! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/22/2009 20:54'!
rocketFinished: id withVelocity: velocity

	Transcript show: 'rocket ', id asString ,' finished ', velocity asString; cr.

	velocity < self explodeLimit
		ifTrue: [ "loses"
			self player: id lost: self pointsForLosing.
			(self otherRocketFor: id) finished ifTrue: [ self switchToState: #stopped ].
		]
		ifFalse: [ "won"
			(self otherRocketFor: id) finished
				ifTrue: [ "won normal"
					self player: id won: self pointsForWinning
				]
				ifFalse: [ "'double'"
					self
						player: id won: self pointsForDoubleWinning;
						playerGameOver: (self otherPlayerFor: id)
				].
			self switchToState: #stopped.
		].! !

!PhdExLunarLander methodsFor: 'velocities' stamp: 'lk 8/21/2009 23:29'!
rocketFor: id

	id = #left ifTrue: [ ^ leftRocket ].
	id = #right ifTrue: [ ^ rightRocket ].! !

!PhdExLunarLander methodsFor: 'rockets landed' stamp: 'lk 8/21/2009 23:21'!
setPlayerPoints: rocketId to: points

	rocketId = #left ifTrue: [ leftPlayer := points ].
	rocketId = #right ifTrue: [ rightPlayer := points ].! !

!PhdExLunarLander methodsFor: 'rocket data' stamp: 'lk 8/22/2009 18:57'!
setRocketData

	leftRocket
		setMotor: ( motorControl motors at: self leftMotor )
		baseInput: ( interfaceKit inputs at: self leftBaseInput )
		controlSensor: ( interfaceKit sensors at: self leftControlSensor ).
	rightRocket
		setMotor: ( motorControl motors at: self rightMotor )
		baseInput: ( interfaceKit inputs at: self rightBaseInput )
		controlSensor: ( interfaceKit sensors at: self rightControlSensor ).! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:31'!
setStateLEDs: anArray
	"#( leftGreen leftYellow leftRed rightGreen rightYellow rightRed )
	If the value is true, enable the LED. Else turn it off."
	
	interfaceKit isAttached ifFalse: [ ^ self ].
	
	(interfaceKit outputs at: self leftGreenLED) value: (anArray at: 1).
	(interfaceKit outputs at: self leftYellowLED) value: (anArray at: 2).
	(interfaceKit outputs at: self leftRedLED) value: (anArray at: 3).
	(interfaceKit outputs at: self rightGreenLED) value: (anArray at: 4).
	(interfaceKit outputs at: self rightYellowLED) value: (anArray at: 5).
	(interfaceKit outputs at: self rightRedLED) value: (anArray at: 6).! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 23:12'!
setStateLeds: anArray for: rocketId
	"rocketId is either #left or #right
	#( [id]Green [id]Yellow [id]Red )
	If the value is true, enable the LED. Else turn it off."
	
	interfaceKit isAttached ifFalse: [ ^ self ].
	
	(interfaceKit outputs at: (self greenLEDFor: rocketId)) value: (anArray at: 1).
	(interfaceKit outputs at: (self yellowLEDFor: rocketId)) value: (anArray at: 2).
	(interfaceKit outputs at: (self redLEDFor: rocketId)) value: (anArray at: 3).! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/22/2009 20:44'!
showControllerInStartingPositionFor: rocketId

	(self controllerInStartingPosition: rocketId)
		ifTrue: [ self setStateLeds: #( true false false ) for: rocketId ]
		ifFalse: [ self setStateLeds: #( false false true ) for: rocketId ].! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/22/2009 20:44'!
showControllersInStartingPosition

	self
		showControllerInStartingPositionFor: #left;
		showControllerInStartingPositionFor: #right.! !

!PhdExLunarLander methodsFor: 'info text' stamp: 'lk 8/22/2009 21:14'!
showInfo: aString

	lcd isAttached ifTrue: [
		lcd textLCD displayString: aString inRow: 2.
		"aString size > lcd textLCD columns
			ifTrue: [ lcd textLCD startScrollingRow: 2 ]
			ifFalse: [ lcd textLCD stopScrollingRow: 2 ]."
	].! !

!PhdExLunarLander methodsFor: 'points' stamp: 'lk 8/22/2009 21:01'!
showPlayerPoints

	lcd isAttached ifTrue: [
		lcd textLCD
			displayString: 'left: ', leftPlayer asString, '  right: ', rightPlayer asString
			inRow: 1.
	].! !

!PhdExLunarLander methodsFor: 'velocities' stamp: 'lk 8/21/2009 23:08'!
showRocketVelocities

	self showRocketVelocitiesFor: #left.
	self showRocketVelocitiesFor: #right.! !

!PhdExLunarLander methodsFor: 'velocities' stamp: 'lk 8/22/2009 20:23'!
showRocketVelocitiesFor: id

	| rocket |
	rocket := (self rocketFor: id).

	rocket finished ifFalse: [
		rocket velocity >= self minUncertaintyLimit ifTrue: [
			self setStateLeds: #( true false false ) for: id.
			^ self
		].
		rocket velocity <= self maxUncertaintyLimit ifTrue: [
			self setStateLeds: #( false false true ) for: id.
			^ self
		].
		self setStateLeds: #( false true false ) for: id.
	].! !

!PhdExLunarLander methodsFor: 'accessing' stamp: 'lk 8/22/2009 14:25'!
start

	state := #stopped.	
	self
		startLedStateProcess;
		openPhidgets;
		registerController.! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 21:20'!
startLedStateProcess

	[ self ledStateProcess ] fork.! !

!PhdExLunarLander methodsFor: 'rocket actions' stamp: 'lk 8/22/2009 19:12'!
startRockets

	[ leftRocket start ] fork.
	[ rightRocket start ] fork.! !

!PhdExLunarLander methodsFor: 'accessing' stamp: 'lk 8/22/2009 19:35'!
stop

	self
		stopRockets;
		switchToState: #initialized; "will stop LED state process"
		unregisterController;
		closePhidgets.! !

!PhdExLunarLander methodsFor: 'rocket actions' stamp: 'lk 8/21/2009 22:31'!
stopRockets

	leftRocket stop.
	rightRocket stop.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/22/2009 21:06'!
switchToState: newState

	state := newState.
	
	state = #stopped ifTrue: [
		self
			unregisterAttachEvents;
			unregisterDetachEvents;
			unregisterResetDoneEvents;
			unregisterCountDownDoneEvent;
			unregisterFlyingDoneEvents;
			stopRockets;
			showPlayerPoints;
			showInfo: 'Please reset rockets to start a new round.'.
	].
	state = #waitForAttach ifTrue: [
		self
			registerOnAllAttachedReset;
			registerOnDetachStopRockets;
			showInfo: 'Please attach the Phidgets.'.
	].
	state = #resetting ifTrue: [
		Transcript show: 'switch to state resetting'; cr.
		self
			unregisterAttachEvents;
			setRocketData;
			registerResetDoneEvents;
			resetRockets;
			showInfo: 'Resetting rockets ...'.
	].
	state = #reset ifTrue: [
		self
			unregisterResetDoneEvents;
			ifPlayersReadyStartRound;
			showInfo: 'Rockets are reset. Choose ''start round'' for next round.'.
	].
	state = #waitForControllersInStartingPosition ifTrue: [
		self ifControllersInStartingPositionStartRound; "if now not statisfied, any sensor change will test this again."
			showInfo: 'Please set your controllers to starting position to start round.'.
	].
	state = #countDown ifTrue: [
		self registerCountDownDoneEvent.
	].
	state = #roundStarted ifTrue: [
		self
			unregisterCountDownDoneEvent;
			registerFlyingDoneEvents;
			startRockets;
			showInfo: 'Good luck!! =)'.
	].! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:49'!
unregisterAttachEvents

	motorControl
		removeActionsWithReceiver: self
		forEvent: #Attach.
	interfaceKit
		removeActionsWithReceiver: self
		forEvent: #Attach.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/20/2009 22:16'!
unregisterController

	interfaceKit
		removeActionsWithReceiver: self
		forEvent: #SensorChange.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:18'!
unregisterCountDownDoneEvent

	self
		removeActionsWithReceiver: self
		forEvent: #CountDownDone.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:17'!
unregisterDetachEvents

	motorControl
		removeActionsWithReceiver: self
		forEvent: #Detach.
	interfaceKit
		removeActionsWithReceiver: self
		forEvent: #Detach.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/22/2009 14:20'!
unregisterFlyingDoneEvents

	leftRocket
		removeActionsWithReceiver: self
		forEvent: #FlyingDone.
	rightRocket
		removeActionsWithReceiver: self
		forEvent: #FlyingDone.! !

!PhdExLunarLander methodsFor: 'events and states' stamp: 'lk 8/21/2009 21:18'!
unregisterResetDoneEvents

	leftRocket
		removeActionsWithReceiver: self
		forEvent: #ResetDone.
	rightRocket
		removeActionsWithReceiver: self
		forEvent: #ResetDone.! !

!PhdExLunarLander methodsFor: 'leds' stamp: 'lk 8/21/2009 23:15'!
yellowLEDFor: rocketId

	rocketId = #left ifTrue: [ ^ self leftYellowLED ].
	rocketId = #right ifTrue: [ ^ self rightYellowLED ].! !

Object subclass: #PhdExLunarLanderRocket
	instanceVariableNames: 'id motor baseInput controlSensor isInverted state velocity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExLunarLanderRocket class methodsFor: 'as yet unclassified' stamp: 'lk 8/22/2009 13:29'!
newAs: rocketId inverted: aBoolean

	^ self new
		id: rocketId;
		inverted: aBoolean;
		yourself! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 13:56'!
deleteData

	motor := nil.
	baseInput := nil.
	controlSensor := nil.! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 19:43'!
finished

	^ state ~= #flying! !

!PhdExLunarLanderRocket methodsFor: 'events' stamp: 'lk 8/22/2009 14:23'!
flyingDone: lastVelocity

	self
		triggerEvent: #FlyingDone
		withArguments: { id. lastVelocity }.! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 19:15'!
getCurrentControlSensorValue

	^ controlSensor
		ifNotNil: [ controlSensor value ]
		ifNil: [ 0 ].! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 19:58'!
gravity

	^ -5! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 13:05'!
id: rocketId

	id := rocketId.! !

!PhdExLunarLanderRocket methodsFor: 'initialization' stamp: 'lk 8/22/2009 13:49'!
initialize

	super initialize.
	
	state := #initialized.
	velocity := 0.! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 13:16'!
inverted: aBoolean

	isInverted := aBoolean.! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 18:58'!
isOnTheGround

	^ baseInput value! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 18:58'!
isReset

	^ state = #reset! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 20:04'!
raiseVelocityBy: anInteger

	velocity := velocity + anInteger.
	self updateVelocity.! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 20:35'!
reset

	| delay |
	
	state ~= #dataSet ifTrue: [ ^ self ].
	state := #resetting.

	delay := Delay forMilliseconds: 100.
	
	"if already on the ground, wind up until not on ground any more to prevent drooping ropes"
	(state = #resetting and: [self isOnTheGround]) ifTrue: [
		self setVelocity: 50.
		[ state = #resetting and: [ self isOnTheGround ] ] whileTrue: [ delay wait ].
		self stopMotor.
	].
	
	"go down to the ground"
	(state = #resetting and: [self isOnTheGround]) ifFalse: [
		self setVelocity: -50.
		[ state = #resetting and: [ self isOnTheGround not ] ] whileTrue: [ delay wait ].
		self stopMotor.
	].

	"wind up"
	self setVelocity: 50.
	100 timesRepeat: [
		state = #resetting ifFalse: [ self stopMotor. ^ self ].
		delay wait.
	].
	self stopMotor.
	
	state := #reset.
	self resetDone.! !

!PhdExLunarLanderRocket methodsFor: 'events' stamp: 'lk 8/22/2009 14:03'!
resetDone

	self triggerEvent: #ResetDone.! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 14:23'!
setMotor: aPhdCrCompMotor
baseInput: aPhdCrCompDigitalInput
controlSensor: aPhdCrCompAnalogSensor

	state ~= #initialized ifTrue: [ ^ self ].

	motor := aPhdCrCompMotor.
	baseInput := aPhdCrCompDigitalInput.
	controlSensor := aPhdCrCompAnalogSensor.
	
	state := #dataSet.! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 14:06'!
setVelocity: newVelocity

	velocity := newVelocity.
	self updateVelocity.! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 20:07'!
start

	| delay lastVelocity |
	
	state ~= #reset ifTrue: [ ^ self ].
	state := #flying.
	
	delay := Delay forMilliseconds: 200.
	
	self setVelocity: 0.
	
	[ state = #flying and: [ self isOnTheGround not ] ] whileTrue: [
	
		"motor and gravity"
		self raiseVelocityBy:
			self getCurrentControlSensorValue // 100 +
			self gravity.
		
		delay wait.
	].

	lastVelocity := velocity.
	self
		stop;
		flyingDone: lastVelocity.! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 13:59'!
stop

	self
		stopMotor;
		deleteData.
	state := #initialized.! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 14:07'!
stopMotor

	self setVelocity: 0.! !

!PhdExLunarLanderRocket methodsFor: 'actions' stamp: 'lk 8/22/2009 20:04'!
updateVelocity

	motor ifNotNil: [ | velocityToSet |
		velocityToSet := isInverted
			ifTrue: [ velocity negated ]
			ifFalse: [ velocity ].
		motor velocity: velocityToSet.
	].! !

!PhdExLunarLanderRocket methodsFor: 'accesssing' stamp: 'lk 8/22/2009 19:15'!
velocity

	^ velocity! !

Object subclass: #PhdExSqueakLogoLCD
	instanceVariableNames: 'lcdPhidget'
	classVariableNames: 'CustomCharCollection'
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExSqueakLogoLCD class methodsFor: 'as yet unclassified' stamp: 'fs 8/13/2009 19:54'!
customCharCollection

	CustomCharCollection ifNotNil: [ ^ CustomCharCollection].
	
	CustomCharCollection := Array new: 8.

	^ CustomCharCollection
		at: 1 put: (PhdCrCustomChar newWith: 542284 and: 4360);
		at: 2 put: (PhdCrCustomChar newWith: 0 and: 420684);
		at: 3 put: (PhdCrCustomChar newWith: 0 and: 210342);
		at: 4 put: (PhdCrCustomChar newWith: 50470 and: 4162);
		at: 5 put: (PhdCrCustomChar newWith: 397063 and: 8339);
		at: 6 put: (PhdCrCustomChar newWith: 60720 and: 24);
		at: 7 put: (PhdCrCustomChar newWith: 552513 and: 3);
		at: 8 put: (PhdCrCustomChar newWith: 221308 and: 2201)
	! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/12/2009 22:25'!
initialize

	super initialize.
	
	lcdPhidget := PhdCrTextLCD new.! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/13/2009 19:53'!
setCustomChars
		
	lcdPhidget textLCD setAllCustomChars: self class customCharCollection.! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/13/2009 19:39'!
setText

	lcdPhidget textLCD
		displayString: 
			PhdCrCustomChar first, 
			PhdCrCustomChar second, 
			PhdCrCustomChar third, 
			PhdCrCustomChar fourth,
			'  I''m a' 
		inRow: 1;
		displayString: 
			PhdCrCustomChar fifth, 
			PhdCrCustomChar sixth, 
			PhdCrCustomChar seventh, 
			PhdCrCustomChar eighth,
			'  Squeeeeaker!!' 
		inRow: 2.! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/12/2009 22:39'!
start

	lcdPhidget openAny.
	lcdPhidget on: #Attach do: [
		self 
			setCustomChars;
			setText.
	].! !

!PhdExSqueakLogoLCD methodsFor: 'as yet unclassified' stamp: 'fs 8/12/2009 22:34'!
stop

	lcdPhidget close.! !

Object subclass: #PhdExStarTrekLCD
	instanceVariableNames: 'running lcdPhidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExStarTrekLCD commentStamp: 'pt 7/15/2009 08:31' prior: 0!
This is an example for the usage of a Text LCD Phidget (usually bundled with an Interface Kit).

Execute the following line after you've attached the LCD Phidget.

"Create a new example object."
lcd := PhdExTextLCD new.

"Start the demo - this opens a connection to the phidget"
lcd start.

"Stop the demo - this closes the connection to the phidget"
lcd stop.!

!PhdExStarTrekLCD methodsFor: 'initialize' stamp: 'pt 7/15/2009 07:50'!
initialize

	super initialize.
	running := false.
	lcdPhidget := PhdCrTextLCD new.! !

!PhdExStarTrekLCD methodsFor: 'events' stamp: 'fs 7/24/2009 15:52'!
starTrekProcess
	"Let some stars slide along the display.
	 Insiders know where to find it in Star Treck ;)"

	| t |
	t := '          '.
	[ running ] whileTrue: [
		10 to: 1 by: -2 do: [ :i | | k |
			t at: i put: $*; at: i - 1 put: $*.
			k := t , t reverse.
			lcdPhidget textLCD
				displayString: k inRow: 1;
				displayString: k	inRow: 2.
			(Delay forMilliseconds: 500) wait.
			t at: i put: Character space; at: i - 1 put: Character space ] ].! !

!PhdExStarTrekLCD methodsFor: 'events' stamp: 'fs 7/24/2009 15:51'!
start

	running := true.
	lcdPhidget openAny.
	[ self starTrekProcess ] fork! !

!PhdExStarTrekLCD methodsFor: 'events' stamp: 'fs 7/24/2009 15:51'!
stop

	running := false.
	lcdPhidget close.! !

Object subclass: #PhdExTagMap
	instanceVariableNames: 'sensors tags knownTagHandler newTagHandler tagLostHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExTagMap commentStamp: 'pt 7/6/2009 16:53' prior: 0!
This is an example about how to use the RFID Phidget from Squeak with PhidgetLab.
This one is more generci than PhdExRFIDPaint and need further programming.

If you give an instance of this class a sensor from an RFID Phidget (PhdCrRFID) it associates an object to each RFID tag. You can specefy the behavior for new/known tags. You might also specify what happens if a tag was found/lost.
To make an instance forget known tags, execute: self forgetTags.

For an example look at PhdExRFIDPaint.!

PhdExTagMap subclass: #PhdExRFIDPaint
	instanceVariableNames: 'rfidPhidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Examples'!

!PhdExRFIDPaint commentStamp: 'pt 7/6/2009 16:54' prior: 0!
This is an example about how to use the Phidget RFID with Squeak/PhidgetLab.

Execute the following in a workspace:
map := PhdExRFIDPaint new. "opens a new instance of this example"
map start. "starts the 'game' and opens a connection to the Phidget"
map stop. "closes the connection - recommended if you want to use this Phidget later. If the connection stays open, the Phidget can not be used somewhere else."!

!PhdExRFIDPaint methodsFor: 'events' stamp: 'fs 7/24/2009 15:47'!
attach
	"Enables the antenna, if I was attached."
	
	rfidPhidget rfidSensor isAntennaOn: true! !

!PhdExRFIDPaint methodsFor: 'initialization' stamp: 'pt 7/6/2009 16:40'!
initialize

	super initialize.
	rfidPhidget := PhdCrRFID new.
	
	"set own handlers that cannot be overwritten."
	newTagHandler := [ :tagString |
		SketchMorph openEditor ].
	
	knownTagHandler := [ :tagString :oldInfo | 
		oldInfo
			openInWorld;
			visible: true;
			yourself ].
	
	tagLostHandler := [ :tagString :oldInfo | 
		oldInfo
			visible: false;
			yourself ].

	"register events"
	rfidPhidget on: #Attach	do: [ self attach ]! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
knownTagHandler

	^ knownTagHandler! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:45'!
knownTagHandler: aBlock
	"cannot be set in this subclass."! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
newTagHandler

	^ newTagHandler! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:45'!
newTagHandler: aBlock
	"cannot be set in this subclass."! !

!PhdExRFIDPaint methodsFor: 'as yet unclassified' stamp: 'pt 7/6/2009 16:39'!
start
	"opens a connection to an RFID phidget."
	
	rfidPhidget openAny.
	self
		addSensor: rfidPhidget rfidSensor;
		attach.! !

!PhdExRFIDPaint methodsFor: 'as yet unclassified' stamp: 'pt 7/3/2009 14:22'!
stop
	"close a connection to an RFID phidget."
	
	rfidPhidget close.! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
tagLostHandler

	^ tagLostHandler! !

!PhdExRFIDPaint methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:45'!
tagLostHandler: aBlock
	"cannot be set in this subclass."! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:31'!
addSensor: anRFIDSensor

	sensors add: (PhdExListItem new: anRFIDSensor).
	anRFIDSensor on: #TagFound do: [ self tagFound: anRFIDSensor ].
	anRFIDSensor on: #TagLost do: [ self tagLost: anRFIDSensor ].! !

!PhdExTagMap methodsFor: 'initialization' stamp: 'pt 7/3/2009 14:40'!
forgetTags
	"clears my tag cache"
	
	tags := Dictionary new.! !

!PhdExTagMap methodsFor: 'initialization' stamp: 'lk 7/3/2009 11:58'!
initialize

	super initialize.
	
	sensors := LinkedList new.
	tags := Dictionary new.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:59'!
knownTagHandler

	knownTagHandler ifNil: [ 
		knownTagHandler := [ :tagString :oldInfo | oldInfo ] ].

	^ knownTagHandler! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/2/2009 23:40'!
knownTagHandler: aBlock
	"aBlock must take 1 argument: the String that represents the found tag we already know.
	When evaluated it must return an object to be associated with that tag."
	
	knownTagHandler := aBlock.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:59'!
newTagHandler

	newTagHandler ifNil: [
		newTagHandler := [ :tagString | tagString ] ].

	^ newTagHandler! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/2/2009 23:39'!
newTagHandler: aBlock
	"aBlock must take 1 argument: the String that represents the newly found tag..
	When evaluated it must return an object to be associated with that tag."
	
	newTagHandler := aBlock.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/2/2009 22:24'!
setTag: aString info: anObject

	tags at: aString asSymbol put: anObject.! !

!PhdExTagMap methodsFor: 'tag handler' stamp: 'lk 7/3/2009 12:05'!
tagFound: anRFIDSensor
	"anRFIDSensor is the sensor which found the tag"
	
	| tagString tagInfo |
	tagString := anRFIDSensor lastTag.
	
	tagInfo := (tags includesKey: tagString asSymbol)
		ifTrue:  [
			self knownTagHandler
				value: tagString
				value: (tags at: tagString asSymbol) ]
		ifFalse: [ self newTagHandler value: tagString ].
	
	self setTag: tagString info: tagInfo.! !

!PhdExTagMap methodsFor: 'tag handler' stamp: 'lk 7/3/2009 11:30'!
tagLost: anRFIDSensor
	"anRFIDSensor is the sensor which lost the tag"
	
	| tagString tagInfo |
	tagString := anRFIDSensor lastTag.
	
	tagInfo := (tags includesKey: tagString asSymbol)
		ifTrue:  [
			tagLostHandler
				value: tagString
				value: (tags at: tagString asSymbol) ]
		ifFalse: [ nil ].
	
	self setTag: tagString info: tagInfo.! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 12:00'!
tagLostHandler

	tagLostHandler ifNil: [ 
		tagLostHandler := [ :tagString :oldInfo | oldInfo ] ].

	^ tagLostHandler! !

!PhdExTagMap methodsFor: 'accessing' stamp: 'lk 7/3/2009 11:31'!
tagLostHandler: aBlock
	"aBlock must take 2 arguments: the String that represents the lost tag, and its currently set info.
	When evaluated it must return an object to be associated with that tag."
	
	tagLostHandler := aBlock.! !
